+{
   locale_version => 0.88,
   entry => <<'ENTRY', # for DUCET v6.1.0
0063 0068 ; [.15EB.0020.0002.0063] # <LATIN SMALL LETTER C, LATIN SMALL LETTER H>
0043 0068 ; [.15EB.0020.0007.0043] # <LATIN CAPITAL LETTER C, LATIN SMALL LETTER H>
0043 0048 ; [.15EB.0020.0008.0043] # <LATIN CAPITAL LETTER C, LATIN CAPITAL LETTER H>
0067 0062 ; [.1677.0020.0002.0067] # <LATIN SMALL LETTER G, LATIN SMALL LETTER B>
0047 0062 ; [.1677.0020.0007.0047] # <LATIN CAPITAL LETTER G, LATIN SMALL LETTER B>
0047 0042 ; [.1677.0020.0008.0047] # <LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER B>
0067 0068 ; [.1678.0020.0002.0067] # <LATIN SMALL LETTER G, LATIN SMALL LETTER H>
0047 0068 ; [.1678.0020.0007.0047] # <LATIN CAPITAL LETTER G, LATIN SMALL LETTER H>
0047 0048 ; [.1678.0020.0008.0047] # <LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER H>
0067 0077 ; [.1679.0020.0002.0067] # <LATIN SMALL LETTER G, LATIN SMALL LETTER W>
0047 0077 ; [.1679.0020.0007.0047] # <LATIN CAPITAL LETTER G, LATIN SMALL LETTER W>
0047 0057 ; [.1679.0020.0008.0047] # <LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER W>
1ECB      ; [.16B3.0020.0002.1ECB] # LATIN SMALL LETTER I WITH DOT BELOW
0069 0323 ; [.16B3.0020.0002.1ECB] # LATIN SMALL LETTER I WITH DOT BELOW
1ECA      ; [.16B3.0020.0008.1ECA] # LATIN CAPITAL LETTER I WITH DOT BELOW
0049 0323 ; [.16B3.0020.0008.1ECA] # LATIN CAPITAL LETTER I WITH DOT BELOW
006B 0070 ; [.16E5.0020.0002.006B] # <LATIN SMALL LETTER K, LATIN SMALL LETTER P>
004B 0070 ; [.16E5.0020.0007.004B] # <LATIN CAPITAL LETTER K, LATIN SMALL LETTER P>
004B 0050 ; [.16E5.0020.0008.004B] # <LATIN CAPITAL LETTER K, LATIN CAPITAL LETTER P>
006B 0077 ; [.16E6.0020.0002.006B] # <LATIN SMALL LETTER K, LATIN SMALL LETTER W>
004B 0077 ; [.16E6.0020.0007.004B] # <LATIN CAPITAL LETTER K, LATIN SMALL LETTER W>
004B 0057 ; [.16E6.0020.0008.004B] # <LATIN CAPITAL LETTER K, LATIN CAPITAL LETTER W>
1E45      ; [.1735.0020.0002.1E45] # LATIN SMALL LETTER N WITH DOT ABOVE
006E 0307 ; [.1735.0020.0002.1E45] # LATIN SMALL LETTER N WITH DOT ABOVE
1E44      ; [.1735.0020.0008.1E44] # LATIN CAPITAL LETTER N WITH DOT ABOVE
004E 0307 ; [.1735.0020.0008.1E44] # LATIN CAPITAL LETTER N WITH DOT ABOVE
006E 0077 ; [.1736.0020.0002.006E] # <LATIN SMALL LETTER N, LATIN SMALL LETTER W>
004E 0077 ; [.1736.0020.0007.004E] # <LATIN CAPITAL LETTER N, LATIN SMALL LETTER W>
004E 0057 ; [.1736.0020.0008.004E] # <LATIN CAPITAL LETTER N, LATIN CAPITAL LETTER W>
006E 0079 ; [.1737.0020.0002.006E] # <LATIN SMALL LETTER N, LATIN SMALL LETTER Y>
004E 0079 ; [.1737.0020.0007.004E] # <LATIN CAPITAL LETTER N, LATIN SMALL LETTER Y>
004E 0059 ; [.1737.0020.0008.004E] # <LATIN CAPITAL LETTER N, LATIN CAPITAL LETTER Y>
1ECD      ; [.1757.0020.0002.1ECD] # LATIN SMALL LETTER O WITH DOT BELOW
006F 0323 ; [.1757.0020.0002.1ECD] # LATIN SMALL LETTER O WITH DOT BELOW
1ECC      ; [.1757.0020.0008.1ECC] # LATIN CAPITAL LETTER O WITH DOT BELOW
004F 0323 ; [.1757.0020.0008.1ECC] # LATIN CAPITAL LETTER O WITH DOT BELOW
1ED9      ; [.1757.0020.0002.1ECD][.0000.003C.0002.0302] # LATIN SMALL LETTER O WITH CIRCUMFLEX AND DOT BELOW
1ED8      ; [.1757.0020.0008.1ECC][.0000.003C.0002.0302] # LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND DOT BELOW
1EE3      ; [.1757.0020.0002.1ECD][.0000.0068.0002.031B] # LATIN SMALL LETTER O WITH HORN AND DOT BELOW
1EE2      ; [.1757.0020.0008.1ECC][.0000.0068.0002.031B] # LATIN CAPITAL LETTER O WITH HORN AND DOT BELOW
0073 0068 ; [.17D9.0020.0002.0073] # <LATIN SMALL LETTER S, LATIN SMALL LETTER H>
0053 0068 ; [.17D9.0020.0007.0053] # <LATIN CAPITAL LETTER S, LATIN SMALL LETTER H>
0053 0048 ; [.17D9.0020.0008.0053] # <LATIN CAPITAL LETTER S, LATIN CAPITAL LETTER H>
1EE5      ; [.181C.0020.0002.1EE5] # LATIN SMALL LETTER U WITH DOT BELOW
0075 0323 ; [.181C.0020.0002.1EE5] # LATIN SMALL LETTER U WITH DOT BELOW
1EE4      ; [.181C.0020.0008.1EE4] # LATIN CAPITAL LETTER U WITH DOT BELOW
0055 0323 ; [.181C.0020.0008.1EE4] # LATIN CAPITAL LETTER U WITH DOT BELOW
1EF1      ; [.181C.0020.0002.1EE5][.0000.0068.0002.031B] # LATIN SMALL LETTER U WITH HORN AND DOT BELOW
1EF0      ; [.181C.0020.0008.1EE4][.0000.0068.0002.031B] # LATIN CAPITAL LETTER U WITH HORN AND DOT BELOW
ENTRY
};
